import sys
import os
import re

from PyQt5 import QtWidgets, QtGui, QtCore    #, uic, QtQml, QtQuick
from PyQt5.QtWidgets import QFileDialog, QMessageBox
from PyQt5.QtCore import Qt, QTimer

import json

import GroupsFrame, LabelFrame, iws_defs #, GraphFrame

class TabsWindow(QtWidgets.QMainWindow):

    def __init__(self, config_file, parent=None):
        super(TabsWindow, self).__init__(parent)

        self.parent = parent
        self.label_frames = []
        self.name = ''
        self.layout(config_file)

        self.close_button.clicked.connect(self.do_close)

        self.all_labels = {}


    def do_close(self):
        self.parent.child_window_closed(self.name)
        self.close()

    def layout(self, config_file):

        with open(config_file) as f:
            json_data = json.load(f)

        #frame = self.make_parameter_frame(json_data['tabs'])
        pframe = self.make_frame(json_data['tabs'])
        bframe = self.make_button_frame()
        layout = QtWidgets.QVBoxLayout()
        layout.addWidget(bframe)
        layout.addWidget(pframe)
        widget = QtWidgets.QWidget()
        widget.setLayout(layout)
        self.setCentralWidget(widget)
        name = json_data['name']
        self.setWindowTitle(name)
        self.name = name
        self.show()
        

    def make_button_frame(self):
        frame = QtWidgets.QFrame()
        self.close_button = QtWidgets.QPushButton('Close')
        layout = QtWidgets.QHBoxLayout()
        layout.addStretch(1)
        layout.addWidget(self.close_button)
        frame.setLayout(layout)
        return frame


    def make_frame(self, json_data):
        #frame = QtWidgets.QFrame()
        #frame.setStyleSheet("background-color:blue;")
        
        pframe = self.make_parameter_frame(json_data)

        #layout = QtWidgets.QHBoxLayout()
        #layout.addWidget(pframe)
        #frame.setLayout(layout)
        return pframe

    def make_parameter_frame(self, jdata):
        tab_frame = QtWidgets.QFrame()
        tab_widget = QtWidgets.QTabWidget()
        font = self.font()
        font.setBold(True)
        tab_widget.setFont(font)

        n = 0
        for i in jdata:
            tab_name = i['name']
            ltab = GroupsFrame.GroupsFrame(i['groups'], n, tab_widget, self.parent) # maybe should be parent, not self?
            ltab.group_name = tab_name
            #ltab.setStyleSheet("background-color:yellow")
            tab_widget.addTab(ltab, tab_name)
            n += 1
            self.label_frames.append(ltab)
            
        layout = QtWidgets.QHBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(tab_widget)
        tab_frame.setLayout(layout)
        return tab_frame


    def update_tm(self, parameter_string):
        for frame in self.label_frames:
            #frame.update_tm(parameter_string)
            if frame.update_tm(parameter_string):
                break

    
    def update_tab_colour(self):
        for frame in self.label_frames:
            frame.update_tab_colour()
