import re

"""
Package iws contains functions used throughout the IWS TCTM Monitor
"""

def get_json_labels(json_data):
    """ json_data = {name:name,labels:[{mib_name:mn,mib_id:mi,source:src,unit:v} """
    print("hello world get_json_labels")
    labels = []
    json_labels = json_data["labels"]
    for i in json_labels:
        #print(i['mib_name'] + ' ' + i['mib_id'] + ' ' + i['source'] + ' ' + i['unit'])
        labels.append((i['mib_name'], i['mib_id'], i['source'], i['unit']))

    return labels

def get_alt_json_labels(json_data):
    """ json_data = {name:name,labels:[{mib_name:mn,mib_id:mi,source:src,unit:v} """
    """ json_data = {name:n,groups:[{row:r,column:c,labels:[{mib_name:mn,mib_id:mi,source:src,unit:v}]}]} """
    print("hello world get_alt_json_labels")

    labels = []
    groups = json_data['groups']
    #print('GROUPS')
    #print(groups)
    #json_labels = json_data["labels"]
    for group in groups:
        print('GROUP')
        print(group)
        #group_labels = groups[group]
        json_labels = group['labels']
        for i in json_labels:
            #print(i['mib_name'] + ' ' + i['mib_id'] + ' ' + i['source'] + ' ' + i['unit'])
            labels.append((i['mib_name'], i['mib_id'], i['source'], i['unit']))

    return labels


## Reads a file of Euclid TM parameters
# @param path A string containing the full path to the parameter file
# @returns An array of parameter tuples
#
# Each tuple is in the form: (MIB_name_string, MIB_ID_string, source_hk_string, unit_string)
# Eg: ('VIS_CDPU_5V_CPU', 'VDGT2712', 'rt_slow', 'V')
def read_parameters(path):
    print("hello world read_parameters")
    """ Reads a file of Euclid TM parameters.
    The parameter is a string containing the full path to the parameter file. 
    Returns an array of parameter tuples (mib name, mib is, source packet, unit). """

    blank = re.compile('^$')
    comment = re.compile('^\?')
    minus = re.compile('^---')
    pluses = re.compile('^\+\+\+')

    labels = []
    
    try:
        with open(path, 'r') as f:
            for line in f:
                #print(line)
                if blank.match(line):
                    #print("blank line")
                    continue
                if comment.match(line):
                    #print("comment")
                    continue
                if minus.match(line):
                    #print("minus")
                    continue
                if pluses.match(line):
                    #print("plus")
                    continue
#                labels.append(line.rstrip(os.linesep))
#                labels.append(line.strip("\r"))
                labels.append(tuple(line.split()))
#                labels.append("".join(line.split('\n')))
    except Exception as e:
        #print("Exception thrown reading file: %d" % e.errno, e)
        print("read_parameters: Exception thrown reading file: %s" % line)
    finally:
        f.close()
        
    return labels

def read_parameters_as_dictionary(path):

    labels = {}
    
    try:
        with open(path, 'r') as f:
            for line in f:
                list = line.split()
                key = list[1]
                labels[key] = {}
    except Exception as e:
        print("Exception thrown reading file: %d" % e.errno, e)
    finally:
        f.close()
        
    return labels
