from PyQt5 import QtCore, QtWidgets, QtGui
from PyQt5.QtCore import Qt, QMimeData, QRegExp
from PyQt5.QtWidgets import QTreeWidgetItem, QFrame, QFileDialog, QMessageBox
from PyQt5.QtGui import QDrag, QFont, QRegExpValidator

import sys
import re

import json

import iws_defs
import chooser_defs
#from lib.Config import chooser_defs

from lib.Lists.Lists import ListItem, ChildListItem, ParentListItem, List, MasterList
#import ListItem
#import ParentListItem
#import List
#import MasterList

from lib.Config import PageFrame, PlotFrame, MPlotFrame
#import PageFrame
#import PlotFrame

# Nasty hard wired numbers for the window size
XTOP = 0
YTOP = 0
WIDTH = 1550
HEIGHT = 1200


#choices = ['system', 'system1', 'packets']
choices = ['system', 'system sorted', 'packets']

system_names = ['CDPU', 'OBS-G', 'PMCU', 'ROE', 'RPSU']
system_tabs = {'CDPU'  : chooser_defs.cdpu_parameters,
               'OBS-G' : chooser_defs.obsg_parameters,
               'PMCU'  : chooser_defs.pmcu_parameters,
               'ROE'   : chooser_defs.roe_parameters,
               'RPSU'  : chooser_defs.rpsu_parameters}

system1_tabs = {'CDPU'  : chooser_defs.cdpu_parameters1,
                'OBS-G' : chooser_defs.obsg_parameters1,
                'PMCU'  : chooser_defs.pmcu_parameters1,
                'ROE'   : chooser_defs.roe_parameters1,
                'RPSU'  : chooser_defs.rpsu_parameters1}

packet_names = ['ST_SLOW','ST_SUPERFAST', 'RT_SLOW', 'RT_FAST', 'RT_SUPERFAST']
packet_tabs = {'ST_SLOW'      : chooser_defs.st_slow_parameters,
               'ST_SUPERFAST' : chooser_defs.st_superfast_parameters,
               'RT_SLOW'      : chooser_defs.rt_slow_parameters,
               'RT_FAST'      : chooser_defs.rt_fast_parameters,
               'RT_SUPERFAST' : chooser_defs.rt_superfast_parameters}
    



drag_in_progress = False
drop_completed = False

class ConfigWindow(QtWidgets.QMainWindow):

    def __init__(self):
        super(ConfigWindow, self).__init__()
        print("__init__")
        self.trees = []

        self.system_trees = {'CDPU'  : [],
                             'OBS-G' : [],
                             'PMCU'  : [],
                             'ROE'   : [],
                             'RPSU'  : []}
        self.system1_trees = {'CDPU'  : [],
                             'OBS-G' : [],
                             'PMCU'  : [],
                             'ROE'   : [],
                             'RPSU'  : []}
        self.packet_trees = {'RT_FAST'      : [],
                             'RT_SLOW'      : [],
                             'RT_SUPERFAST' : [],
                             'ST_SLOW'      : [],
                             'ST_SUPERFAST' : []}

        self.parameter_lists = {'system'  : self.system_trees,
                                'system sorted' : self.system1_trees,
                                'packets' : self.packet_trees}

        self.type_choice = 'system'

        self.current_record_box = None

        self.initUi()

        # Variables for creating json output
        self.config = {}
        self.tab_names = {}
        self.plot_names = {}
        self.tabs = []
        self.plots = []

        
    def initUi(self):
        #print("initUi")

        self.setObjectName("MainWindow")
        self.setWindowTitle("Make Config")

        centralWidget = QtWidgets.QWidget(self)
        centralWidget.setObjectName("centralWidget")
        centralWidget.setAccessibleName("centralWidget")
        ###centralWidget.setStyleSheet("background-color:pink;")

#        mainMenu = self.menuBar()
#        fileMenu = mainMenu.addMenu('File')
#        quitAct = QtWidgets.QAction('Quit', self, checkable = False)
#        fileMenu.addAction(quitAct)

        self.statusbar = QtWidgets.QStatusBar(self)
        self.statusbar.setObjectName("statusbar")
        self.statusbar.showMessage('Message in statusbar.')
        self.setStatusBar(self.statusbar)

        top_frame = QtWidgets.QFrame()

        # DEBUG
        #top_frame.setStyleSheet("background-color:orange;")

        top_frame.setFrameStyle(QtWidgets.QFrame.StyledPanel)
        top_frame_layout = QtWidgets.QVBoxLayout()
        #top_frame_layout.setContentsMargins(0, 0, 0, 0)

        main_frame = QtWidgets.QFrame()

        # DEBUG
        #main_frame.setStyleSheet("background-color:lightblue;")

        main_frame.setFrameStyle(QtWidgets.QFrame.StyledPanel)
        main_frame_layout = QtWidgets.QHBoxLayout()

        self.tabWidgetFrame = self.createTabWidgetFrame()
        self.tabWidgetFrame.setMinimumSize(QtCore.QSize(350, 100))
        self.page_frame = self.create_pages_frame()

        record_frame = self.create_record_frame()
        control_frame = self.create_control_frame()

        main_frame_layout.addWidget(record_frame)
        main_frame_layout.addWidget(self.tabWidgetFrame, stretch = 1)
        main_frame_layout.addWidget(self.page_frame)
        main_frame.setLayout(main_frame_layout)

        top_frame_layout.addWidget(control_frame)
        top_frame_layout.addWidget(main_frame)
        top_frame.setLayout(top_frame_layout)

        centralWidget.setLayout(top_frame_layout)

        self.configuration_dirty = False
        self.page_window_dirty = False
        self.plot_window_dirty = False

        self.setCentralWidget(centralWidget)


    def create_control_frame(self):
        frame = QtWidgets.QFrame()
        frame.setObjectName("control_frame")

        #DEBUG
        #frame.setStyleSheet("background-color:yellow;")

        frame.setFrameStyle(QtWidgets.QFrame.StyledPanel)

        quit_button = QtWidgets.QPushButton("Quit")
        quit_button.clicked.connect(self.quit_clicked)

        self.save_button = QtWidgets.QPushButton("Save")
        self.save_button.clicked.connect(self.save_clicked)
        self.save_button.setEnabled(False)

        read_button = QtWidgets.QPushButton("Read")
        read_button.clicked.connect(self.read_clicked)

        l = QtWidgets.QLabel("Name:")
        self.configuration_name_ledit = QtWidgets.QLineEdit()
        rx = QRegExp("[A-Za-z]+[A-Za-z0-9_]*")
        v = QRegExpValidator(rx, self)
        self.configuration_name_ledit.setValidator(v)
        self.configuration_name_ledit.editingFinished.connect(self.nameEdited)
        self.configuration_name_ledit.textChanged.connect(self.nameEdited)
        ###print(f"MIN SIZE = {self.configuration_name_ledit.minimumSizeHint()}")
        ### NASTY!
        self.configuration_name_ledit.setFixedWidth(300)

        self.combo_index = 0
        self.options_combo = QtWidgets.QComboBox()
        self.options_combo.addItem('Configuration')
        self.options_combo.addItem('Page Window')
        self.options_combo.addItem('Plot Window')
        self.options_combo.addItem('Multi Plot Window')
        self.options_combo.activated.connect(self.handleActivated)

        hbox = QtWidgets.QHBoxLayout()
        #hbox.addStretch(1)
        hbox.addWidget(read_button)
        hbox.addWidget(self.save_button)
        hbox.addWidget(self.options_combo)
        hbox.addWidget(l)
        hbox.addWidget(self.configuration_name_ledit)
        hbox.addStretch(1)
        hbox.addWidget(quit_button)
        frame.setLayout(hbox)
        return frame


    # https://stackoverflow.com/questions/46361675/pyqt-enforcing-sizehint-dimensions-on-two-widget-app-with-layout-manager
    def create_record_frame(self):
        frame = QtWidgets.QFrame()

        # DEBUG
        #frame.setStyleSheet("background-color:pink;")

        frame.setFrameStyle(QtWidgets.QFrame.StyledPanel)
        layout = QtWidgets.QVBoxLayout()

        button_frame = QtWidgets.QFrame()
        button_frame_layout = QtWidgets.QHBoxLayout()
        #button_frame_layout.setContentsMargins(0, 0, 0, 0)

        self.delete_button = QtWidgets.QPushButton('Delete')
        self.edit_button = QtWidgets.QPushButton('Edit')

        self.delete_button.setEnabled(False)
        self.edit_button.setEnabled(False)

        self.delete_button.clicked.connect(self.delete_button_clicked)
        self.edit_button.clicked.connect(self.edit_button_clicked)

        button_frame_layout.addWidget(self.delete_button)
        button_frame_layout.addWidget(self.edit_button)
        button_frame.setLayout(button_frame_layout)

        page_lists_frame = QtWidgets.QGroupBox("Tabs")

        # DEBUG
        #page_lists_frame.setStyleSheet("background-color:lightgreen;")

        page_lists_frame_layout = QtWidgets.QVBoxLayout()
        page_lists_frame_layout.setContentsMargins(0, 0, 0, 0)

        self.page_list = QtWidgets.QListWidget()
        self.page_list.setObjectName('page_list')
###        self.page_list.clicked.connect(self.page_list_clicked)
        # Connect signals
        self.page_list.currentItemChanged.connect(self.current_item_changed)

        page_lists_frame_layout.addWidget(self.page_list)
        page_lists_frame.setLayout(page_lists_frame_layout)

        plot_lists_frame = QtWidgets.QGroupBox("Plots")

        # DEBUG
        #plot_lists_frame.setStyleSheet("background-color:lightblue;")

        plot_lists_frame_layout = QtWidgets.QVBoxLayout()
        plot_lists_frame_layout.setContentsMargins(0, 0, 0, 0)
        self.plot_list = QtWidgets.QListWidget()
        self.plot_list.setObjectName('plot_list')
###        self.plot_list.clicked.connect(self.plot_list_clicked)
        self.plot_list.currentItemChanged.connect(self.plot_list_current_item_changed)

        plot_lists_frame_layout.addWidget(self.plot_list)
        plot_lists_frame.setLayout(plot_lists_frame_layout)

        layout.addWidget(page_lists_frame, stretch = 1)
        layout.addWidget(plot_lists_frame, stretch = 1)
        layout.addWidget(button_frame)
        layout.setContentsMargins(0, 0, 0, 0)

        frame.setLayout(layout)
        return frame


    def create_pages_frame(self):
        tabWidget = QtWidgets.QTabWidget()

        # DEBUG
        #tabWidget.setStyleSheet("background-color:lightgreen;")

        tabWidget.setObjectName("pagesTabWidget")

        page_tab = QtWidgets.QWidget(tabWidget)
        page_tab.setObjectName("pageTab")
        self.page_frame1 = PageFrame.PageFrame(parent=self)
        page_tab.setLayout(self.page_frame1.layout())

        plot_tab = QtWidgets.QWidget(tabWidget)
        plot_tab.setObjectName("plotTab")
        self.plot_frame1 = PlotFrame.PlotFrame(parent=self)
        plot_tab.setLayout(self.plot_frame1.layout())

        mplot_tab = QtWidgets.QWidget(tabWidget)
        mplot_tab.setObjectName("mplotTab")
        self.mplot_frame1 = MPlotFrame.MPlotFrame(parent=self)
        mplot_tab.setLayout(self.mplot_frame1.layout())

        tabWidget.addTab(page_tab, "Tab")
        tabWidget.addTab(plot_tab, "Plot")
        tabWidget.addTab(mplot_tab, "MPlot")
        return tabWidget



    def nameEdited(self):
        name = self.configuration_name_ledit.text()
        enable = True if name != '' else False
        self.save_button.setEnabled(enable)
        
        self.configuration_name_ledit.adjustSize()


    def handleActivated(self, index):
        self.combo_index = index


    def quit_clicked(self):
        quitButtonReply = QtWidgets.QMessageBox.question(self,
                                                         'PyQt5 message', "Are you sure you want to quit Make Config?",
                                                         QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No, QtWidgets.QMessageBox.No)
        if quitButtonReply == QtWidgets.QMessageBox.Yes:
            self.close()
            exit(1)

            
    def dump_json(self, where, what):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        suffixes = 'json file (*.json)'

        filename, _ = QFileDialog.getSaveFileName(self, "QFileDialog.getSaveFileName()", where, suffixes, options=options)
        if filename:
            print(f"filename = {filename}")
            if filename.endswith('.json'):
                pass
            else:
                filename += '.json'
                print(f"filename = {filename}")
            with open(filename, 'w') as outfile:
                json.dump(self.config, outfile)
                self.statusbar.showMessage(what + " saved")
        else:
            self.statusbar.showMessage("")


    def save_configuration(self, name, ntabs, nplots):
        if (ntabs == 0) and (nplots == 0):
            self.statusbar.showMessage("Require at least one tab page or one plot page")
            return
        self.config['name'] = name
        self.config['ntabs'] = ntabs
        self.config['ngraphs'] = nplots
        self.config['nwindows'] = 0
        self.config['tabs'] = self.tabs
        self.config['graphs'] = self.plots
        self.config['windows'] = []
        self.dump_json(iws_defs.defaults_directory, 'Configuration')
        self.configuration_dirty = False


    def save_page_window(self, name, ntabs):
        if ntabs == 0:
            self.statusbar.showMessage("Require at least one tab")
            return
        self.config['name'] = name
        self.config['ntabs'] = ntabs
        self.config['tabs'] = self.tabs
        self.dump_json(iws_defs.windows_directory, 'Page Window')
        self.page_window_dirty = False
        pass


    def save_plot_window_common(self, name, nplots, where, what):
        self.config['name'] = name
        self.config['ngraphs'] = nplots
        self.config['graphs'] = self.plots
        self.dump_json(where, what)
        self.plot_window_dirty = False


    def save_plot_window(self, name, nplots):
        if nplots == 0:
            self.statusbar.showMessage("Require at least one plot")
            return
        self.save_plot_window_common(name, nplots, iws_defs.plots_directory, 'Plot Window')
        #self.config['name'] = name
        #self.config['ngraphs'] = nplots
        #self.config['graphs'] = self.plots
        #self.dump_json(iws_defs.plots_directory, 'Plot Window')
        #self.plot_window_dirty = False
        pass


    def save_multi_plot_window(self, name, nplots):
        if nplots == 0:
            self.statusbar.showMessage("Require at least one plot")
            return
        if nplots > 12:
            self.statusbar.showMessage("Too many (> 12) plots defined for multi plot")
            return
        self.save_plot_window_common(name, nplots, iws_defs.multi_plots_directory, 'Plot Window')
        #self.config['name'] = name
        #self.config['ngraphs'] = nplots
        #self.config['graphs'] = self.plots
        #self.dump_json(iws_defs.multi_plots_directory, 'Plot Window')
        #self.plot_window_dirty = False
        pass


    def save_clicked(self):
        self.statusbar.showMessage("")
        name = self.configuration_name_ledit.text()
        if name == '':
            self.statusbar.showMessage("A name is required")
            return
        
        ntabs = len(self.tabs)
        nplots = len(self.plots)
        type = self.options_combo.itemText(self.combo_index)
        if type == 'Configuration':
            self.save_configuration(name, ntabs, nplots)
        elif type == 'Page Window':
            self.save_page_window(name, ntabs)
        elif type == 'Plot Window':
            self.save_plot_window(name, nplots)
            #self.save_plot_window(iws_defs.plots_directory, name, nplots, type)
        else:
            self.save_multi_plot_window(name, nplots)
            #self.save_plot_window(iws_defs.multi_plots_directory, name, nplots, type)


    def load_configuration(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        suffixes = 'json file (*.json)'
        filename, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()", iws_defs.defaults_directory, suffixes, options=options)
        if filename:
            #self.clear()
            with open(filename) as f:
                data = json.load(f)
                self.configuration_name_ledit.setText(data['name'])
                self.config['name'] = data['name']
                self.config['ntabs'] = data['ntabs']
                self.config['ngraphs'] = data['ngraphs']
                self.config['nwindows'] = data['nwindows']
                self.config['tabs'] = data['tabs']
                self.config['graphs'] = data['graphs']
                self.config['windows'] = data['windows']
                self.page_frame1.clear()
                self.clear_tab_list()
                n = 0
                for t in self.config['tabs']:
                    self.add_page_from_read(t)
                    if n == 0:
                        self.page_frame1.populate_group(t)
                    n += 1
                self.plot_frame1.clear()
                self.clear_plot_list()
                n = 0
                for g in self.config['graphs']:
                    self.add_plot_from_read(g)
                    if n == 0:
                        self.plot_frame1.populate_group(g)
                    n += 1
        self.configuration_dirty = True


    def load_page_window(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        suffixes = 'json file (*.json)'
        filename, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()", iws_defs.windows_directory, suffixes, options=options)
        if filename:
            #self.clear()
            with open(filename) as f:
                data = json.load(f)
                self.configuration_name_ledit.setText(data['name'])
                self.config['name'] = data['name']
                self.config['ntabs'] = data['ntabs']
#                self.config['ngraphs'] = data['ngraphs']
#                self.config['nwindows'] = data['nwindows']
                self.config['tabs'] = data['tabs']
#                self.config['graphs'] = data['graphs']
#                self.config['windows'] = data['windows']
                self.page_frame1.clear()
                self.clear_tab_list()
                n = 0
                for t in self.config['tabs']:
                    self.add_page_from_read(t)
                    if n == 0:
                        self.page_frame1.populate_group(t)
                    n += 1
 #               self.plot_frame1.clear()
 #               self.clear_plot_list()
 #               n = 0
 #               for g in self.config['graphs']:
 #                   self.add_plot_from_read(g)
 #                   if n == 0:
 #                       self.plot_frame1.populate_group(g)
 #                   n += 1
        self.configuration_dirty = True


    def load_plot_window(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        suffixes = 'json file (*.json)'
        filename, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()", iws_defs.plots_directory, suffixes, options=options)
        if filename:
            #self.clear()
            with open(filename) as f:
                data = json.load(f)
                self.configuration_name_ledit.setText(data['name'])
                self.config['name'] = data['name']
#                self.config['ntabs'] = data['ntabs']
###                self.config['nplots'] = data['nplots']
                self.config['ngraphs'] = data['ngraphs']
#                self.config['nwindows'] = data['nwindows']
#                self.config['tabs'] = data['tabs']
                self.config['graphs'] = data['graphs']
#                self.config['windows'] = data['windows']
#                self.page_frame1.clear()
#                self.clear_tab_list()
#                n = 0
#                for t in self.config['tabs']:
#                    self.add_page_from_read(t)
#                    if n == 0:
#                        self.page_frame1.populate_group(t)
#                    n += 1
                self.plot_frame1.clear()
                self.clear_plot_list()
                n = 0
                for g in self.config['graphs']:
                    self.add_plot_from_read(g)
                    if n == 0:
                        self.plot_frame1.populate_group(g)
                    n += 1
        self.configuration_dirty = True


    def load_multi_plot_window(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        suffixes = 'json file (*.json)'
        filename, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()", iws_defs.multi_plots_directory, suffixes, options=options)
        if filename:
            #self.clear()
            with open(filename) as f:
                data = json.load(f)
                self.configuration_name_ledit.setText(data['name'])
                self.config['name'] = data['name']
#                self.config['ntabs'] = data['ntabs']
###                self.config['nplots'] = data['nplots']
                self.config['ngraphs'] = data['ngraphs']
#                self.config['nwindows'] = data['nwindows']
#                self.config['tabs'] = data['tabs']
                self.config['graphs'] = data['graphs']
#                self.config['windows'] = data['windows']
#                self.page_frame1.clear()
#                self.clear_tab_list()
#                n = 0
#                for t in self.config['tabs']:
#                    self.add_page_from_read(t)
#                    if n == 0:
#                        self.page_frame1.populate_group(t)
#                    n += 1
                self.plot_frame1.clear()
                self.clear_plot_list()
                n = 0
                for g in self.config['graphs']:
                    self.add_plot_from_read(g)
                    if n == 0:
                        self.plot_frame1.populate_group(g)
                    n += 1
        self.configuration_dirty = True


    def read_clicked(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        suffixes = 'json file (*.json)'

        print(f"Options is {self.options_combo.currentText()}")
        selection = self.options_combo.currentText()
        if selection == 'Configuration':
            self.load_configuration()
        elif selection == "Page Window":
            self.load_page_window()
        elif selection == "Plot Window":
            self.load_plot_window()
        else:
            self.load_multi_plot_window()


    def createTab(self, type):
        font = QtGui.QFont()
        font.setPointSize(9)
        #font.setBold(True)
        font.setWeight(50)

        names = system_names if ((type == 'system') or (type == 'system sorted')) else packet_names
        #tabs =  system_tabs  if type == 'system' else packet_tabs
        tabs =  system_tabs  if type == 'system' else system1_tabs if type == 'system sorted' else packet_tabs

        tabWidget = QtWidgets.QTabWidget()
        tabWidget.setFont(font)
        tabWidget.setObjectName(f"tabWidget_{type}")
###        tab_number = 0
        for name in names:
            layout = QtWidgets.QHBoxLayout()
            tab = QtWidgets.QWidget()
            tab.setFont(font)
            tab.setObjectName(name + '_tab')

            ###list = self.createList(name, tabs[name], type, tab_number)
            list = self.createList(name, tabs[name], type)

            self.parameter_lists[type][name].append(list)
            ##self.parameter_trees[type][name].append(list)
            #self.parameter_trees[type][name] = list

            #self.parameter_trees[type][name] = self.createTree(tab, name, tabs[name], type)
            
            layout.addWidget(list, stretch = 1)
            tab.setLayout(layout)

            tabWidget.addTab(tab, name)
###            tab_number += 1

        tabWidget.setCurrentIndex(0)
        return tabWidget


    def createTabWidgetFrame(self):
#        frame = QtWidgets.QFrame()
        #frame.setStyleSheet("background-color:lightblue;")
#        frame.setFrameStyle(QtWidgets.QFrame.StyledPanel)
        #frame.setContentsMargins(0, 0, 0, 0)

        #layout = QtWidgets.QHBoxLayout()
        #layout.addStretch(1)
        #frame.setLayout(layout)

        self.masterTabWidget = QtWidgets.QTabWidget()
        #self.masterTabWidget.setMinimumSize(QtCore.QSize(iws_defs.TEXT_FIELD_MINIMUM_SIZE + 140, 100))
        #self.masterTabWidget.setMinimumSize(QtCore.QSize(350, 100))
        #self.masterTabWidget.setStyleSheet("background-color:yellow;")

        #tabWidget = QtWidgets.QTabWidget()
        #tabWidget.setStyleSheet("background-color:lightgreen;")

        font = QtGui.QFont()
        font.setPointSize(9)
        #font.setBold(True)
        font.setWeight(50)
        #tabWidget.setFont(font)
        #tabWidget.setObjectName("tabWidget")

        self.masterTabWidget.setFont(font)
        self.masterTabWidget.setObjectName("masterTabWidget")

        for type in choices:
            self.masterTabWidget.addTab(self.createTab(type), type)
        
        """
        type = 'system'
        names = system_names if type == 'system' else packet_names
        tabs =  system_tabs  if type == 'system' else packet_tabs

        tabWidget = QtWidgets.QTabWidget()
        #tabWidget.setStyleSheet("background-color:lightgreen;")
        tabWidget.setFont(font)
        tabWidget.setObjectName(f"tabWidget_{type}")
        for name in names:
            layout = QtWidgets.QHBoxLayout()
            tab = QtWidgets.QWidget()
            tab.setObjectName(name + '_tab')
            list = self.createList(name, tabs[name], type)
            self.parameter_lists[type][name].append(list)
            layout.addWidget(list, stretch = 1)
            tab.setLayout(layout)
            tabWidget.addTab(tab, name)

        self.masterTabWidget.setCurrentIndex(0)
        self.masterTabWidget.addTab(tabWidget, type)

        
        type = 'packets'
        names = system_names if type == 'system' else packet_names
        tabs =  system_tabs  if type == 'system' else packet_tabs
        tabWidget = QtWidgets.QTabWidget()
        #tabWidget.setStyleSheet("background-color:lightgreen;")
        tabWidget.setFont(font)
        tabWidget.setObjectName(f"tabWidget_{type}")
        for name in names:
            layout = QtWidgets.QHBoxLayout()
            tab = QtWidgets.QWidget()
            tab.setObjectName(name + '_tab')
            list = self.createList(name, tabs[name], type)
            self.parameter_lists[type][name].append(list)
            layout.addWidget(list, stretch = 1)
            tab.setLayout(layout)
            tabWidget.addTab(tab, name)
            
#        tabWidget.setCurrentIndex(0)
        self.masterTabWidget.addTab(tabWidget, type)
        """

        return self.masterTabWidget


    def createList(self, label, parameter_file, type):

#        list = MasterList.MasterList(label, parent = self)
        list = MasterList(label, parent = self)
        list.resize(200,100)
        #list.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

        seen_parent = False
        blank = re.compile('^$')
        comment = re.compile('^\?')
        alt_comment = re.compile('^;')
        minus = re.compile('^---')
        pluses = re.compile('^\+\+\+')
        
        try:
            with open(parameter_file, 'r') as f:
                sub_tab_number = 0
                row = 0
                for line in f:
                    # txt
                    #items = line.split()
                    # csv
                    items = line.rstrip('\n').split(',')
                    #print(f"{items}")
                    if blank.match(line):
                        seen_parent = False
                        continue
                    if comment.match(line):
                        continue
                    if alt_comment.match(line):
                        continue
                    if minus.match(line):
                        seen_parent = True
                        parent = ParentListItem(items[1], items[2], items[3], items[4], row)
                        list.addItem(parent)
                        row += 1
                        continue
                    if pluses.match(line):
                        seen_parent = True
                        parent = ParentListItem(items[1], items[2], items[3], items[4], row)
                        list.addItem(parent)
                        row += 1
                        continue
                    if seen_parent:
                        child = ChildListItem(items[0], items[1], items[2], items[3], row)
                        #child.main_tab = main_tab_number
                        child.sub_tab = sub_tab_number
                        child.set_parent(parent.name)
                        child.set_parent_id(parent.id)
                        #orphan.main_tab = main_tab_number
                        child.sub_tab = sub_tab_number
                        list.addItem(child)
                    else:
                        orphan = ChildListItem(items[0], items[1], items[2], items[3], row)
                        #orphan.main_tab = main_tab_number
                        orphan.sub_tab = sub_tab_number
                        list.addItem(orphan)
                        seen_parent = False
                row += 1
            sub_tab_number += 1
        except Exception as e:
            print(f"EXCEPTION {e}")
        finally:
            #f.close()
            pass

        return list


    def make_labels(self, box):

        labels = []
        if box.count() == 0:
            return labels
        for r in range(box.count()):
            dict = {}
            items = re.split('\s+', box.item(r).text())
            name = items[0]
            id = items[1]
            src = items[2]
            unit = items[3]
            dict['mib_name'] = name
            dict['mib_id'] = id
            dict['source'] = src
            dict['unit'] = unit
            labels.append(dict)
        return labels


    def make_group(self, box, n):
        r, c = divmod(n, 6)
        dict = {}
        dict['row'] = r
        dict['column'] = c
        dict['labels'] = self.make_labels(box)
        return dict


    def add_page_from_read(self, json_tab):
        self.tabs.append(json_tab)
        name = json_tab['name']
        self.page_list.addItem(name)
        self.tab_names[name] = True
#        self.page_list.setCurrentItem(None)                                                                              
        self.page_list.setCurrentRow(0)
        self.toggle_buttons(True)
#        self.page_list.update(set_to_index=[0], scroll_to_index=0)


    def add_page(self, name, list_box_array):
        try:
            if self.tab_names[name] == True:
                self.statusbar.showMessage("Tab " + name + " is already defined")
                return False
        except Exception as e:
            pass
        self.statusbar.showMessage("Adding tab " + name + " to the configuration")
        groups = []
        n = 0
        for b in list_box_array:
            groups.append(self.make_group(b, n))
            n += 1
        dict = {}
        dict['name'] = name
        dict['groups'] = groups
        self.tabs.append(dict)
        self.page_list.addItem(name)
        self.tab_names[name] = True
        self.page_list.setCurrentItem(None)
        #self.toggle_buttons(False)
        self.toggle_buttons(True)
        return True


    def add_plot_from_read(self, json_plot):
        self.plots.append(json_plot)
        name = json_plot['name']
        self.plot_list.addItem(name)
        self.plot_names[name] = True
#        self.plot_list.setCurrentItem(None)                                                                              
        self.plot_list.setCurrentRow(0)
        self.toggle_buttons(True)


    # minv and maxv are strings, we want real numbers
    def add_plot(self, name, list_box, minv, maxv, unit):
        try:
            if self.plot_names[name] == True:
                self.statusbar.showMessage("Plot " + name + " is already defined")
                return False
        except Exception as e:
            pass
        min_value = 0.0
        max_value = 0.0
        try:
            min_value = float(minv)
            max_value = float(maxv)
        except Exception as e:
            print(f"An error occurred converting minv {minv} or maxv {maxv}")
        self.statusbar.showMessage("Adding plot " + name + " to the configuration")
        yaxis1 = {}
        yaxis1['label'] = unit
        yaxis1['min'] = min_value
        yaxis1['max'] = max_value
        group = self.make_labels(list_box)
        dict = {}
        dict['name'] = name
        dict['naxes'] = 1
        dict['yaxis1'] = yaxis1
        dict['yaxis2'] = {}
        dict['labels'] = group
        self.plots.append(dict)
        self.plot_list.addItem(name)
        self.plot_names[name] = True
        self.plot_list.setCurrentItem(None)
        #self.toggle_buttons(False)
        self.toggle_buttons(True)
        return True


    def delete_from_array(self, name, array):
        for d in array:
            if name == d['name']:
                array.remove(d)
                break
        pass


    def clear_tab_list(self):
###        print(f"clear_tab_list: {self.tab_names}")
        for name in self.tab_names:
            self.delete_from_array(name, self.tabs)
            self.page_list.takeItem(0)
        self.tab_names.clear()


    def clear_plot_list(self):
###        print(f"clear_plot_list: {self.plot_names}")
        for name in self.plot_names:
            self.delete_from_array(name, self.plots)
            self.plot_list.takeItem(0)
        self.plot_names.clear()


    def delete_button_clicked(self):
        item = self.current_record_box.currentItem()
        name = item.text()
        row = self.current_record_box.currentRow()
        self.current_record_box.takeItem(row)
        if self.current_record_box.objectName() == 'page_list':
            del self.tab_names[name]
            self.delete_from_array(name, self.tabs)
            #self.page_frame1.clear_all()
        else:
            del self.plot_names[name]
            self.delete_from_array(name, self.plots)
        if (self.page_list.count() == 0) and (self.plot_list.count() == 0):
            self.toggle_buttons(False)
        pass

    
    def edit_button_clicked(self):
        pass


    def toggle_buttons(self, mode):
        self.delete_button.setEnabled(mode)
        self.edit_button.setEnabled(mode)


    def page_list_clicked(self, qmodelindex):
        item = self.page_list.currentItem()
        row  = self.page_list.currentRow()
        self.current_record_box = self.page_list
        self.toggle_buttons(True)
        self.plot_list.setCurrentItem(None)
        print(f"page_list_clicked {item} at row {row} model = {qmodelindex.row()}")
        pass


    def current_item_changed(self, current, previous):
 #       previous_row = self.page_list.indexFromItem(previous).row()
        current_row  = self.page_list.indexFromItem(current).row()
        if previous == None:
            previous_item = ""
        else:
            previous_item = previous.text()
        print(f"currentItemChanged: {previous_item} to  {self.page_list.indexFromItem(current).row()}")
        #self.page_frame1.clear()
        self.page_frame1.clear_all()
        try:
            self.page_frame1.populate_group(self.tabs[current_row])
        except Exception as e:
            pass
        self.current_record_box = self.page_list


    def plot_list_clicked(self, qmodelindex):
        item = self.plot_list.currentItem()
        row  = self.plot_list.currentRow()
        self.current_record_box = self.plot_list
        self.toggle_buttons(True)
        self.page_list.setCurrentItem(None)
        pass


    def plot_list_current_item_changed(self, current, previous):
 #       previous_row = self.plot_list.indexFromItem(previous).row()
        current_row  = self.plot_list.indexFromItem(current).row()
        if previous == None:
            previous_item = ""
        else:
            previous_item = previous.text()
        print(f"currentItemChanged: {previous_item} to  {self.plot_list.indexFromItem(current).row()}")
        self.plot_frame1.clear()
        try:
            self.plot_frame1.populate_group(self.plots[current_row])
        except IndexError:
            print(f"plot_list_current_item_changed: index error exception: {current_row}")
        except Exception as e:
            print("plot_list_current_item_changed exception")
        self.current_record_box = self.plot_list
    

    # Dummy function to let Lists use this function in PageFrame
    def set_selected_box(self, string):
        pass
