from PyQt5 import QtCore, QtWidgets, QtGui
from PyQt5.QtCore import Qt, QMimeData, QRegExp
from PyQt5.QtWidgets import QTreeWidgetItem, QFrame, QFileDialog, QMessageBox, QCheckBox
from PyQt5.QtGui import QDrag, QFont, QRegExpValidator

import json

import make_config_defs

from lib.Lists.Lists import List, ListItem
#import List
#import ListItem

"""
This class mimics the plot frame in the HKTM monitor and is used to define a new plot window for the monitor.
"""
class PlotFrame(QtWidgets.QFrame):

    def __init__(self, parent=None, subClass=None):
        super(PlotFrame, self).__init__(parent)

        self.parent = parent

        self.combo_index = 0

        master_frame_layout = QtWidgets.QVBoxLayout()

        label_frame  = self.make_label_frame()
        page_frame   = self.make_page_frame(subClass=subClass)
        bottom_frame = self.make_bottom_frame()

        master_frame_layout.addWidget(label_frame)

        if subClass != None:
            tabs_frame = subClass.make_list_frame()
            frame = self.make_new_frame(tabs_frame, page_frame)
            #master_frame_layout.addWidget(tabs_frame)
            master_frame_layout.addWidget(frame)
        else:
            master_frame_layout.addWidget(page_frame)

        master_frame_layout.addWidget(bottom_frame)

        self.setLayout(master_frame_layout)


    def make_new_frame(self, tframe, pframe):
        frame = QtWidgets.QFrame()
        layout = QtWidgets.QHBoxLayout()
        layout.addWidget(pframe)
        layout.addWidget(tframe, QtCore.Qt.AlignLeft)
        layout.addStretch(1)
        frame.setLayout(layout)
        return frame


    def make_bottom_frame(self):
        frame = QtWidgets.QFrame()

        # DEBUG
        #frame.setStyleSheet("background-color:green;")

        layout = QtWidgets.QHBoxLayout()
        self.new_button = QtWidgets.QPushButton("New")
        self.clear_button = QtWidgets.QPushButton("Clear")
        self.add_button = QtWidgets.QPushButton("Add")
        self.add_button.setEnabled(False)
        self.save_button = QtWidgets.QPushButton("Save")
        self.save_button.setEnabled(False)
        self.read_button = QtWidgets.QPushButton("Read")

        self.new_button.clicked.connect(self.new_pressed)
        self.add_button.clicked.connect(self.add_pressed)
        self.clear_button.clicked.connect(self.clear)
        self.save_button.clicked.connect(self.save)
        self.read_button.clicked.connect(self.read)

        layout.addWidget(self.new_button)
        layout.addWidget(self.clear_button)
        layout.addWidget(self.add_button)
        layout.addStretch(1)
        layout.addWidget(self.read_button)
        layout.addWidget(self.save_button)
        frame.setLayout(layout)
        return frame


    def make_page_frame(self, subClass=None):
        frame = QtWidgets.QFrame()
        #frame.setStyleSheet("background-color:yellow;")

        layout = QtWidgets.QGridLayout()
#        layout = None

        self.page_listboxes = []

        """
        name = "Box_0"
        l = List(name, parent = self)
        sp_retain = l.sizePolicy()
        sp_retain.setRetainSizeWhenHidden(True)
        l.setVisible(True)
        l.setSizePolicy(sp_retain)
        self.page_listboxes.append(l)
        layout.addWidget(l, 0, 0)    
        """
        if subClass == None:
 #           layout = QtWidgets.QGridLayout()
            for i in range(12):
                name = "Box_%d" % (i)
                l = List(name,parent=self)
                sp_retain = l.sizePolicy()
                sp_retain.setRetainSizeWhenHidden(True)
                row = 0 if i < 6 else  1
                col = i if i < 6 else i - 6
                if row == 0 and col == 0:
                    l.setVisible(True)
                else:
                    l.setVisible(False)
                l.setSizePolicy(sp_retain)
                self.page_listboxes.append(l)
                layout.addWidget(self.page_listboxes[i], row, col)    
                #print(f"Box size = {l.width()}, {l.height()}")
        else:
            """
            layout = QtWidgets.QHBoxLayout()
            name = "Box_0"
            l = List(name, parent = self)
            l.resize(256, 192)
            sp_retain = l.sizePolicy()
            sp_retain.setRetainSizeWhenHidden(True)
            l.setVisible(True)
            l.setSizePolicy(sp_retain)
            self.page_listboxes.append(l)
            layout.addWidget(l)    
            """
            row = 0
            col = 0
            for i in range(2):
                name = "Box_%d" % (i)
                l = List(name,parent=self)
                l.setMaximumWidth(100)
                sp_retain = l.sizePolicy()
                sp_retain.setRetainSizeWhenHidden(True)
                #row = 0 if i < 3 else  1
                #col = i if i < 3 else i - 3
                #col = i
                if row == 0 and col == 0:
                    l.setVisible(True)
                    #l.setStyleSheet("background-color:red;")
                else:
                    l.setVisible(False)
                    #l.setStyleSheet("background-color:yellow;")
                l.setSizePolicy(sp_retain)
                self.page_listboxes.append(l)
                layout.addWidget(self.page_listboxes[i], row, col, QtCore.Qt.AlignLeft)
                row += 1

        self.list = self.page_listboxes[0]
        frame.setLayout(layout)
        return frame


    def make_label_frame(self):
        frame  = QtWidgets.QFrame()
        layout = QtWidgets.QHBoxLayout()
        # DEBUG
        #label_frame.setStyleSheet("background-color:pink;")
        page_name_label = QtWidgets.QLabel("Plot Name:")
        self.page_name_box = QtWidgets.QLineEdit()
        rx = QRegExp("[A-Za-z]+[A-Za-z0-9_]*")
        v = QRegExpValidator(rx, self)
        self.page_name_box.setValidator(v)
        self.page_name_box.editingFinished.connect(self.nameEdited)
        self.page_name_box.textChanged.connect(self.nameEdited)
        self.page_name_box.setFixedWidth(300)

        self.split_button = QCheckBox("Split Name")

        unit_label = QtWidgets.QLabel("Unit:")
        self.combo = QtWidgets.QComboBox()
        self.combo.addItem("mV")
####        self.combo.addItem("Volts")
        self.combo.addItem("Voltage")
        self.combo.addItem("mA")
        self.combo.addItem("Amps")
        self.combo.addItem("Temperature (C)")
        self.combo.addItem("Temperature (K)")
        self.combo.addItem("None")
        self.combo.activated.connect(self.handleActivated)

        min_label = QtWidgets.QLabel("Min:")
        max_label = QtWidgets.QLabel("Max:")
        self.min_box = QtWidgets.QLineEdit()
        self.max_box = QtWidgets.QLineEdit()
        
        layout.addWidget(page_name_label)
        layout.addWidget(self.page_name_box)
        layout.addWidget(self.split_button)
        layout.addWidget(unit_label)
        layout.addWidget(self.combo)
        layout.addWidget(min_label)
        layout.addWidget(self.min_box)
        layout.addWidget(max_label)
        layout.addWidget(self.max_box)

        layout.addStretch(1)
        frame.setLayout(layout)

        return frame


    def handleActivated(self, index):
        self.combo_index = index


    def clear(self):
        self.page_name_box.setText('')
        self.min_box.setText('')
        self.max_box.setText('')
        nitems = self.list.count()
        for n in range(nitems):
            #print(f"Removing item {n}")
            self.list.takeItem(0)

    def nameEdited(self):
        name = self.page_name_box.text()
        enable = True if name != '' else False
        self.save_button.setEnabled(enable)
        self.add_button.setEnabled(enable)
        pass


    def new_pressed(self):
        pass


    def add_pressed(self):
        name = self.page_name_box.text()
        if name == '':
            self.parent.statusbar.showMessage("Plot name is required")
            return False
        unit = self.combo.itemText(self.combo_index)
        minv = self.min_box.text()
        if minv == '':
            self.parent.statusbar.showMessage("Minimum value is required")
            return False
        maxv = self.max_box.text()
        if maxv == '':
            self.parent.statusbar.showMessage("Maximum value is required")
            return False
        if self.list.count() == 0:
            self.parent.statusbar.showMessage("At least one plot parameter is required")
            return False
        ret = self.parent.add_plot(name, self.list, minv, maxv, unit)
        if ret:
            self.clear()
            return ret


    def save(self):
        name = self.page_name_box.text()
        unit = self.combo.itemText(self.combo_index)
        minv = self.min_box.text()
        maxv = self.max_box.text()
        min_value = 0.0
        max_value = 0.0
        try:
            min_value = float(minv)
            max_value = float(maxv)
        except Exception as e:
            print(f"An error occurred converting minv {minv} or maxv {maxv}")
        # Check name is set
        if self.list.count() == 0:
            self.parent.statusbar.showMessage("At least one parameter is required")
        else:
            dict = {}
            group = []
            group.append(self.parent.make_group(self.list, 0))
            dict['name'] = name
            dict['naxes'] = 1
            yaxis1 = {}
            yaxis1['label'] = unit
            yaxis1['min'] = min_value
            yaxis1['max'] = max_value
            dict['yaxis1'] = yaxis1
            dict['yaxis2'] = {}
            dict['labels'] = group
            
            options = QFileDialog.Options()
            options |= QFileDialog.DontUseNativeDialog
            suffixes = 'json file (*.json)'

            filename, _ = QFileDialog.getSaveFileName(self, "QFileDialog.getSaveFileName()", make_config_defs.plot_tabs_dir, suffixes, options=options)
            if filename:
                print(f"filename is {filename}")
                if filename.endswith('.json'):
                    print(f"(has json) filename = {filename}")
                    pass
                else:
                    filename += '.json'
                    print(f"(needs json) filename = {filename}")
                with open(filename, 'w') as outfile:
                    json.dump(dict, outfile)
        pass


    def read(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        suffixes = 'json file (*.json)'
        filename, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()", make_config_defs.plot_tabs_dir, suffixes, options=options)
        if filename:
            self.clear()
            with open(filename) as f:
                data = json.load(f)
                print(f"PlotFrame : read : {data}")
                self.page_name_box.setText(data['name'])
                yaxis1 = data['yaxis1']
                self.min_box.setText(str(yaxis1['min']))
                self.max_box.setText(str(yaxis1['max']))
                unit = yaxis1['label']
                index = self.combo.findText(unit)
                if index == -1:
                    print("PlotFrame : read : ARRRGHHH!")
                else:
                    self.combo.setCurrentIndex(index)
                labels = data['labels']
                group = labels[0] # [{'row':0,..., 'labels':[]}]
                labels = group['labels']
                for label in labels:
                    item = ListItem(label['mib_name'], label['mib_id'], label['source'], label['unit'], 0)
                    self.list.addItem(item)
        pass


    def populate_group(self, data):
        print(f"populate_group: {data}")
        self.page_name_box.setText(data['name'])
        yaxis1 = data['yaxis1']
        self.min_box.setText(str(yaxis1['min']))
        self.max_box.setText(str(yaxis1['max']))
        unit = yaxis1['label']
        index = self.combo.findText(unit)
        # DEBUG
        print(f"Name {self.page_name_box.text()}")
        print(f"Unit {unit}")
        if index == -1:
                print("PlotFrame : populate_group : ARRRGHHH!")
        else:
            self.combo.setCurrentIndex(index)
            labels = data['labels']
            group = labels[0] # [{'row':0,..., 'labels':[]}]

#        labels = group['labels']

            for label in labels:
                item = ListItem(label['mib_name'], label['mib_id'], label['source'], label['unit'], 0)
                self.list.addItem(item)


    def set_selected_box(self, name):
        pass
